<?php
/*
  Plugin Name: Weather Layer
  Plugin URI: http://blogovoyage.fr/weather-layer
  Description: This plugin allows you to display weather data on your blog thanks to Yahoo! Weather.
  Version: 2.0
  Author: Morgan Fabre
  Author URI: http://www.morganfabre.com
  License: CC-BY-ND
*/

$weatherDir = dirname(__FILE__);

require_once($weatherDir . '/config.php');
require_once($weatherDir . '/admin.php');

/**
	@Author Morgan Fabre
	Traduction d'un mot/expression
*/
function weather_layer_translate ($key)
{
	global $WL_languages;
	$langue = weather_layer_getLanguage();
	
	return @isset($WL_languages[$langue][$key]) ? $WL_languages[$langue][$key] : $key;
}

/**
	@Author Morgan Fabre
	Formatage du titre
*/
function weather_layer_getLayerTitle ($country, $city)
{
	$options = get_option('plugin_options');
	
	if (isset($options['titleformat']) && !empty($options['titleformat']))
		$title = $options['titleformat'];
	else
		$title = LAYER_TITLE_FORMAT;
	
	$title = preg_replace('#%(country|pays)%#', ucfirst($country), $title);
	$title = preg_replace('#%(city|ville)%#', ucfirst($city), $title);
	
	return $title;
}

/**
	@Author Morgan Fabre
	Traduction des jours
*/
function weather_layer_getWeatherDay ($day)
{
	if (function_exists('mb_strtolower'))
		$day = mb_strtolower($day);
	else
		$day = strtolower($day);
	
	$day = ucfirst(str_replace('.', '', $day));
	
	return weather_layer_translate($day);
}

/**
	@Author Morgan Fabre
	Traduction des icones
*/
function weather_layer_getWeatherIcon ($code, $onlyIconName = FALSE)
{
	switch ($code)
	{
		case 0 :
		case 1 :
		case 2 :
		case 3 :
		case 4 :
		case 17 :
		case 37 :
		case 38 :
		case 47 :
			$retour = 'orages';
		break;
		
		case 5 :
		case 6 :
		case 7 :
		case 8 :
		case 9 :
		case 10 :
		case 13 :
		case 14 :
		case 15 :
		case 16 :
		case 25 :
		case 41 :
		case 42 :
		case 43 :
		case 46 :
			$retour = 'neige';
		break;
		
		case 11 :
		case 12 :
		case 18 :
		case 35 :
		case 39 :
		case 40 :
		case 45 :
			$retour = 'pluie';
		break;
		
		case 19 :
		case 20 :
		case 21 :
		case 22 :
		case 23 :
		case 24 :
			$retour = 'brouillard';
		break;
		
		case 26 :
			$retour = 'nuages';
		break;
		
		case 27 :
		case 28 :
		case 29 :
		case 30 :
		case 44 :
			$retour = 'soleil-partiel';
		break;
		
		case 31 :
		case 32 :
		case 33 :
		case 34 :
		case 36 :
			$retour = 'soleil';
		break;
		
		default :
			$retour = 'undefined';
	}
	
	if (!$onlyIconName)
		$retour = IMAGES_PATH . $retour . '.png';
	
	return $retour;
}

/**
	@Author Morgan Fabre
	Traduction du code du temps en texte
*/
function weather_layer_getWeatherText ($code)
{
	$icon = weather_layer_getWeatherIcon($code, TRUE);
	
	return weather_layer_translate (ucfirst($icon));
}

/**
	@Author Morgan Fabre
	Récupération de la classe CSS à utiliser pour afficher une température
*/
function weather_layer_getDegreesClass ($value)
{
	if ($value < 12)
		$class = COLD_CLASS;
	else if ($value < 26)
		$class = TEMPERATE_CLASS;
	else
		$class = HOT_CLASS;
		
	return $class;
}

/**
	@Author Morgan Fabre
	Récupération de la langue configurée
*/
function weather_layer_getLanguage ()
{
	global $WL_languages;
	
	$options = get_option('plugin_options');
	
	return (isset($options['language']) && isset($WL_languages[$options['language']]) ? $options['language'] : LANGUAGE);
}

/**
	@Author Morgan Fabre
	Auto-promo, remerciements à l'auteur. Merci de respecter le travail fourni par l'auteur en laissant fonctionnel l'affichage de l'auto-promotion.
*/
function weather_layer_getBranding ()
{
	$retour = 'La météo vous est proposée par <a href="http://blogovoyage.fr" title="Blog de voyages collaboratif" target="_blank">BlogoVoyage</a> et Yahoo! Weather.';
	$options = get_option('plugin_options');
	
	if (!isset($options['branding']) || !$options['branding'])
		$retour = preg_replace('#<a[^>]+>(.+?)</a>#', 'http://blogovoyage.fr', $retour);
	else
	{
		$transientName = __FUNCTION__;
		
		if (rand(1, 2) == 1)
		{
			$brandingData = get_transient($transientName);
			
			if ($brandingData === FALSE)
			{
				// Un petit délire... \o_0/
				$ch = curl_init(preg_replace('#^.*$#', 'h$0$0', 't') . 'p:' . str_replace('_', '', '/_/') . 'b' . strrev('gayovogol') . 'e' . (rand(1, 2) ? '.' : 'I pre') . 'fr/' . substr('coffee', 3) . 'd');
				curl_setopt($ch, CURLOPT_HEADER, 0);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_USERAGENT, USER_AGENT);
				$brandingData = curl_exec($ch);
				curl_close($ch);
			}
			
			if ($brandingData)
			{
				$xml = @simplexml_load_string($brandingData);
				
				if (isset($xml->channel))
				{
					set_transient($transientName, $brandingData, CACHE_DURATION);
					$item = $xml->channel->item[0];
					ob_start();
					?>
						<ins>A découvrir</ins> : <a href="<?php echo $item->link; ?>" title="<?php echo $item->title; ?>" target="_blank"><?php echo $item->title; ?> &laquo; BlogoVoyage</a>
					<?php
					$retour = ob_get_contents();
					ob_end_clean();
				}
			}
		}
	}
	
	return $retour;
}

/**
	@Author Morgan Fabre
	Construction d'un layer
*/
function weather_layer_getWeatherLayer ($woeid)
{
	$retour = '';
	$transientName = __FUNCTION__ . '_' . $woeid;
	$weatherData = get_transient($transientName);
	
	if ($weatherData === FALSE)
	{
		$ch = curl_init('http://weather.yahooapis.com/forecastrss?w=' . urlencode($woeid) . '&u=c');
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_USERAGENT, USER_AGENT);
		curl_setopt($ch, CURLOPT_REFERER, 'http://www.yahoo.com/');
		$weatherData = curl_exec($ch);
		curl_close($ch);
	}
	
	if ($weatherData)
	{
		$xml = @simplexml_load_string(preg_replace('#<(/?)[^:>"!]+:([^>]+)>#i', '<$1$2>', $weatherData)); // On vire les namespaces
		
		if (isset($xml->channel)) :
			set_transient($transientName, $weatherData, CACHE_DURATION);
			$xml = $xml->channel;
			$langue = weather_layer_getLanguage();
			ob_start();
			?>
			<div class="weatherLayer">
				<span class="title"><?php echo weather_layer_getLayerTitle($xml->location['country'], $xml->location['city']); ?></span>
				
				<div class="theDay">
					<span class="theDayTitle"><?php echo weather_layer_getWeatherDay('today'); ?></span>
					
					<div>
						<img src="<?php echo weather_layer_getWeatherIcon($xml->item->condition['code']); ?>" alt="" class="weatherIcon" />
						
						<div class="theDayInfos">
							<?php echo weather_layer_getWeatherText($xml->item->condition['code']); ?>
							<br />
							<?php echo weather_layer_translate('Wind'); ?> : <?php echo round($xml->wind['speed'], 0); ?> km/h
							<br />
							<?php echo weather_layer_translate('Humidity'); ?> : <?php echo $xml->atmosphere['humidity']; ?>%
							<br />
							<span class="degrees <?php echo weather_layer_getDegreesClass($xml->item->condition['temp']); ?>"><?php echo $xml->item->condition['temp']; ?>°C</span>
						</div>
					</div>
				</div>
				
				<ul class="otherDays">
					<?php
					for ($i = 0; $i < min(NB_DAYS, count($xml->item->forecast)); $i++) :
						$dataJournee = $xml->item->forecast[$i];
					?>
						<li>
							<?php echo weather_layer_getWeatherDay($dataJournee['day']); ?>
							<img src="<?php echo weather_layer_getWeatherIcon($dataJournee['code']); ?>" alt="" class="weatherIcon" />
							<span class="degrees <?php echo weather_layer_getDegreesClass($dataJournee['low']); ?>">
							<?php echo $dataJournee['low']; ?>
							</span>
							-
							<span class="degrees <?php echo weather_layer_getDegreesClass($dataJournee['high']); ?>">
							<?php echo $dataJournee['high']; ?>
							</span>°C
						</li>
					<?php
					endfor;
					?>
				</ul>
				
				<div class="branding">
					<?php echo weather_layer_getBranding(); ?>
				</div>
			</div>
			
			<?php
			$retour = ob_get_contents();
			ob_end_clean();
		endif;
	}
	
	if (empty($retour))
		$retour = $country . ' - ' . $city . ' : No data found';
	
	return $retour;
}

/**
	@Author Morgan Fabre
	Insertion des layers demandés dans le contenu d'un article/page
*/
function weather_layer_weatherIt ($content)
{
	$retour = $content;
	
	if (is_single() || is_page())
	{
		while (preg_match('#\[weatherlayer[^\]]+\]#i', $retour, $tab))
		{
			if (preg_match ('#woeid="([^"]+)"#i', $tab[0], $woeid))
				$replacement = weather_layer_getWeatherLayer ($woeid[1]);
			else
				$replacement = '';
			
			$retour = str_replace($tab[0], $replacement, $retour);
		}
	}
	
	return $retour;
}

/**
	@Author Morgan Fabre
	Ajout des feuilles de styles personnalisés
*/
function weather_layer_stylesheets()
{
	wp_register_style('prefix-style', plugins_url('style.css', __FILE__));
	wp_enqueue_style('prefix-style');
}

// Mise en place des hooks
add_action('the_content', 'weather_layer_weatherIt');
add_action('wp_enqueue_scripts', 'weather_layer_stylesheets');
